; ##############################################################################
; ##############################################################################
; ##############################################################################
EXTERNER_INTERRUPT_KONFIGURIEREN:

;																			
; SLEEP+INT0+INT1 (externer Interrupt) konfigurieren						
;																			
; MCUCR => SE SM2 SM1 SM0 ISC11 ISC10 ISC01 ISC00 							
; SM2 SM1 SM0 Sleep Mode													
;*0   0   0   Idle															
; 0   0   1   ADC Noise Reduction											
; 0   1   0   Power-down													
; 0   1   1   Power-save													
; 1   0   0   Reserved														
; 1   0   1   Reserved														
; 1   1   0   Standby(1)													
;																			
;ISC11 ISC10 Description													
; 0 	0 	The low level of INT1 generates an interrupt request.			
; 0 	1 	Any logical change on INT1 generates an interrupt request.		
; 1 	0 	The falling edge of INT1 generates an interrupt request.		
; 1 	1 	The rising edge of INT1 generates an interrupt request.			
;																			
;ISC01 ISC00 Description													
; 0 	0 	The low level of INT0 generates an interrupt request.			
;*0 	1 	Any logical change on INT0 generates an interrupt request.		
; 1 	0 	The falling edge of INT0 generates an interrupt request.		
; 1 	1 	The rising  edge of INT0 generates an interrupt request.		
;																			
	ldi temp,(1<<SE)|(0<<SM2)|(0<<SM1)|(0<<SM0)|(1<<ISC11)|(0<<ISC10)|(0<<ISC01)|(1<<ISC00) 
	out MCUCR,temp
						
;																			
; INT0+INT1 (externer Interrupt)aktivieren									
;																			
; GICR => INT1 INT0 - - - - IVSEL IVCE 										
; General Interrupt Control													
;																			
	ldi temp, (0<<INT1)|(1<<INT0)|(0<<IVSEL)|(0<<IVCE)   
    out GICR, temp 



; ##############################################################################
; ##############################################################################
; ##############################################################################
EXT_INT0:
	push TEMP		    ; Sichern von "TEMP" im Stack
	in	 TEMP,SREG		; Einlesen des SREG 
	push TEMP	    	; Schreiben von  SREG  im Stack (KOPIE)
	push temp1
; -------------------------------
	; FLANKE_ERMITTELN	
	rcall FLANKE_ERMITTELN
	; FLANKE_AUSWERTEN	
	rcall FLANKE_AUSWERTEN
	; Synchronisation	
	rcall SYNCHRONISATION
; -------------------------------
EXT_INT0_FERTIG:
	pop temp1
	pop	TEMP			; LESEN von SREG vom STACK (KOPIE)
	out	SREG,TEMP		; Wiederherstellen von SREG
	pop	TEMP			; Wiederherstellen von "TEMP" 			
reti
; ##############################################################################
; ##############################################################################
; ##############################################################################
;																				
; nachdem der externe Interrupt ausgelst wurde, wird geprft, wie der 			
; Pegel am PIN ist, also welche Richtung die Flanke des Impulses hat.			
;																				
FLANKE_ERMITTELN:
	; einlesen		
	in temp,PIND
	; BITMUSTER		
	andi temp,0b00000100
	; gleich NULL	
	tst temp
	breq FLANKE_ERMITTELN_FALLEND
;-------------------------------
FLANKE_ERMITTELN_STEIGEND:
	; speichern		
	STS(adr_HF_FLANKE),EINS		; 1=steigend	
	STS(adr_BIT_WERT),NULL
	ret
;-------------------------------
FLANKE_ERMITTELN_FALLEND:
	; speichern		
	STS(adr_HF_FLANKE),NULL		; 0=fallend 	
	STS(adr_BIT_WERT),EINS
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
;																				
; es interessiern nur die Flanken, die in einem Zeitfenster anliegen, 			
; alle anderen Flanken werden ignoriert											
;																				
FLANKE_AUSWERTEN:
	; SYNCHRONISATION aktiv ?
	LDS temp,(adr_HF_SYNCHRONISATION_AKTIV)
	tst temp
	breq FLANKE_AUSWERTEN_w		; nein
	ret
;-------------------------------------------------------------------------------
FLANKE_AUSWERTEN_w:

	; TIMERWERT0 einlesen
	in temp,TCNT0
	; MINIMALWERT		
	ldi temp1,(ZEITFENSTER_MIN)
	; vergleich MIN		
	cp temp,temp1
	brlo FLANKE_UNINTERESSANT

	; MAXIMALWERT		
	ldi temp1,(ZEITFENSTER_MAX)
	; vergleich max		
	cp temp,temp1
	brlo FLANKE_AUSWERTEN_ww
	breq FLANKE_AUSWERTEN_ww
	brsh FLANKE_UNINTERESSANT
;-------------------------------------------------------------------------------
FLANKE_AUSWERTEN_ww:
	; HF_FLANKE_ERKANNT	
	STS(adr_HF_FLANKE_ERKANNT),EINS
	; BITS zuweisen		
	rcall BITS_ZUWEISEN
	; BITS zhlen		
	rcall BITS_ZAEHLEN
	; BITS_AUSWERTEN	
	rcall BITS_AUSWERTEN

	ret
;-------------------------------------------------------------------------------
FLANKE_UNINTERESSANT:

ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
;																				
; den internen TIMER auf den externen Impuls synchronisieren					
;																				
SYNCHRONISATION:
	; SYNCHRONISATION aktiv ?
	LDS temp,(adr_HF_SYNCHRONISATION_AKTIV)
	tst temp
	brne SYNCHRONISATION_w		; ja
	ret
;-------------------------------------------------------------------------------
SYNCHRONISATION_w:
	; ist Flanke steigend ?
	LDS temp,(adr_HF_FLANKE)	; 0=fallend <>0=steigend	
	tst temp
	brne SYNCHRONISATION_START	; ja
	ret
SYNCHRONISATION_START:
	; deaktivieren								
	STS(adr_HF_SYNCHRONISATION_AKTIV),NULL
	; TIMER0 Startwert							
	ldi temp, TIMERWERT0
	out TCNT0,temp  
	; TAKT-PEGEL auf HIGH						
	STS(adr_HF_TAKT_PEGEL),VOLL		
	; an PORT ausgeben							
	out PORTC,VOLL					
	; eventuell anliegender Interrupt lschen	
	ldi temp,(1<<TOV0)
	out TIFR,temp
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
; Die Datenbits 0..2 mssen immer 1 sein, sonst ist Datenpakt ungltig			
BITS_ZUWEISEN:
	; BIT-Nummer laden	
	LDS temp,(adr_BIT_NUMMER)
	; BIT-WERT laden	
	LDS temp1,(adr_BIT_WERT)
BITS_ZUWEISEN_00:
	cpi temp,0					; vergleich	
	brne BITS_ZUWEISEN_01
	STS(adr_DATENBIT_00),temp1	; zuweisen	Synchronisationsbit 0	
	; Fehlerprfung	(muss immer 1 sein)
	rcall BITS_ZUWEISEN_FEHLERPRUEFUNG_EINS
	ret
BITS_ZUWEISEN_01:
	cpi temp,1					; vergleich	
	brne BITS_ZUWEISEN_02
	STS(adr_DATENBIT_01),temp1	; zuweisen	Synchronisationsbit 1	
	; Fehlerprfung	(muss immer 1 sein)
	rcall BITS_ZUWEISEN_FEHLERPRUEFUNG_EINS
	ret
BITS_ZUWEISEN_02:
	cpi temp,2					; vergleich	
	brne BITS_ZUWEISEN_03
	STS(adr_DATENBIT_02),temp1	; zuweisen	Synchronisationsbit 2	
	; Fehlerprfung	(muss immer 1 sein)
	rcall BITS_ZUWEISEN_FEHLERPRUEFUNG_EINS
	ret
BITS_ZUWEISEN_03:
	; GELB an			
	cbi PORTD,LED_D_GELB
	cpi temp,3					; vergleich	
	brne BITS_ZUWEISEN_04
	STS(adr_DATENBIT_03),temp1	; zuweisen	
	ret
BITS_ZUWEISEN_04:
	cpi temp,4					; vergleich	
	brne BITS_ZUWEISEN_05
	STS(adr_DATENBIT_04),temp1	; zuweisen	
	ret
BITS_ZUWEISEN_05:
	cpi temp,5					; vergleich	
	brne BITS_ZUWEISEN_06
	STS(adr_DATENBIT_05),temp1	; zuweisen	
	ret
BITS_ZUWEISEN_06:
	cpi temp,6					; vergleich	
	brne BITS_ZUWEISEN_07
	STS(adr_DATENBIT_06),temp1	; zuweisen	
	ret
BITS_ZUWEISEN_07:
	cpi temp,7					; vergleich	
	brne BITS_ZUWEISEN_08
	STS(adr_DATENBIT_07),temp1	; zuweisen	
	ret
BITS_ZUWEISEN_08:
	cpi temp,8					; vergleich	
	brne BITS_ZUWEISEN_09
	STS(adr_DATENBIT_08),temp1	; zuweisen	
	ret
BITS_ZUWEISEN_09:
	cpi temp,9					; vergleich	
	brne BITS_ZUWEISEN_10
	STS(adr_DATENBIT_09),temp1	; zuweisen	
	ret
BITS_ZUWEISEN_10:
	cpi temp,10					; vergleich	
	brne BITS_ZUWEISEN_11
	STS(adr_DATENBIT_10),temp1	; zuweisen	
	ret
BITS_ZUWEISEN_11:
	ret
;-------------------------------------------------------------------------------
BITS_ZUWEISEN_FEHLERPRUEFUNG_EINS:
	cpi temp1,1	
	brne BITS_ZUWEISEN_FEHLERPRUEFUNG_EINS_ERROR
	ret
BITS_ZUWEISEN_FEHLERPRUEFUNG_EINS_ERROR:
	; BIT-NUMMER NULL			
	STS(adr_BIT_NUMMER),NULL
	; SYNCHRONISATION aktivieren
	STS(adr_HF_SYNCHRONISATION_AKTIV),EINS
	; ROT						
	cbi PORTD,LED_D_ROT	
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
BITS_ZAEHLEN:
	; BIT-NUMMER +1
	LDS temp,(adr_BIT_NUMMER)
	inc temp
	STS(adr_BIT_NUMMER),temp
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
BITS_AUSWERTEN:
	LDS temp,(adr_BIT_NUMMER)
	cpi temp,11
	breq BITS_AUSWERTEN_w
	ret
;-------------------------------------------------------------------------------
BITS_AUSWERTEN_w:
	clr temp1
BITS_AUSWERTEN_D0:
	LDS temp,(adr_DATENBIT_03); DatenBit 0			
	tst temp
	breq BITS_AUSWERTEN_D1
	ori temp1,0b00000001
BITS_AUSWERTEN_D1:
	LDS temp,(adr_DATENBIT_04); DatenBit 1			
	tst temp
	breq BITS_AUSWERTEN_D2
	ori temp1,0b00000010
BITS_AUSWERTEN_D2:
	LDS temp,(adr_DATENBIT_05); DatenBit 2			
	tst temp
	breq BITS_AUSWERTEN_D3
	ori temp1,0b00000100
BITS_AUSWERTEN_D3:
	LDS temp,(adr_DATENBIT_06); DatenBit 3			
	tst temp
	breq BITS_AUSWERTEN_D4
	ori temp1,0b00001000
BITS_AUSWERTEN_D4:
	LDS temp,(adr_DATENBIT_07); DatenBit 4			
	tst temp
	breq BITS_AUSWERTEN_D5
	ori temp1,0b00010000
BITS_AUSWERTEN_D5:
	LDS temp,(adr_DATENBIT_08); DatenBit 5			
	tst temp
	breq BITS_AUSWERTEN_D6
	ori temp1,0b00100000
BITS_AUSWERTEN_D6:
	LDS temp,(adr_DATENBIT_09); DatenBit 6			
	tst temp
	breq BITS_AUSWERTEN_D7
	ori temp1,0b01000000
BITS_AUSWERTEN_D7:
	LDS temp,(adr_DATENBIT_10); DatenBit 7			
	tst temp
	breq BITS_AUSWERTEN_ENDE
	ori temp1,0b10000000
BITS_AUSWERTEN_ENDE:
	; DATENBYTE speichern		
	STS(adr_DATEN_BYTE),temp1
	; BIT-NUMMER NULL			
	STS(adr_BIT_NUMMER),NULL
	; SYNCHRONISATION aktivieren
	STS(adr_HF_SYNCHRONISATION_AKTIV),EINS
	; DATEN_BYTE_GUELTIG		
	STS(adr_DATEN_BYTE_GUELTIG),EINS
	; GELB aus					
	sbi PORTD,LED_D_GELB	; GELB 	
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
