; ##############################################################################
; ##############################################################################
; ##############################################################################
TIMER0_INITIALISIERUNG:
; TIMER0 (8Bit) Vorteiler													
;																			
; CS02 CS01 CS00 	Description												
;	0 	0 	0 		No clock source (Timer/Counter stopped).				
;	0 	0 	1 		clkI/O/		(No prescaling)								
;	0 	1 	0 		clkI/O/8 	(From prescaler)							
; *	0 	1 	1 		clkI/O/64 	(From prescaler)							
; 	1 	0 	0 		clkI/O/256 	(From prescaler)							
; 	1 	0 	1 		clkI/O/1024 (From prescaler)							
;	1 	1 	0 		External clock source on T0 pin. Clock on falling edge.	
;	1 	1 	1 		External clock source on T0 pin. Clock on rising edge.	
	ldi temp,(0<<CS02)|(1<<CS01)|(1<<CS00)
	OUT TCCR0,temp
; TIMER0 Startwert		
	ldi temp, TIMERWERT0
	out TCNT0,temp
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
TIMER_AKTIVIEREN:
;																			
; Timer berlauf-Int aktivieren (Timer/Counter Interrupt Mask)				
;																			
; TIMSK ==> OCIE2 TOIE2 TICIE1 OCIE1A OCIE1B TOIE1 R TOIE0  				
;																			
	ldi temp,(0<<OCIE2)|(0<<TOIE2)|(0<<TICIE1)|(0<<OCIE1A)|(0<<OCIE1B)|(0<<TOIE1)|(1<<TOIE0)
	out TIMSK,temp       
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
; den internen TAKT generieren													
; und den HF-Sender ansteuern													
TIM0_OVF:
	push TEMP		    ; Sichern von "TEMP" im Stack
	in	 TEMP,SREG		; Einlesen des SREG 
	push TEMP	    	; Schreiben von  SREG  im Stack (KOPIE)
	push ZL
	push ZH
; ----------------------------------
	; TIMER0 Startwert				
	ldi temp, TIMERWERT0
	out TCNT0,temp
; ----------------------------------
	; TAKT-PEGEL (HIGH/LOW)			
	LDS temp,(adr_HF_TAKT_PEGEL); laden			
	com temp					; invertieren	
	STS(adr_HF_TAKT_PEGEL),temp	; speichern		
	out PORTC,temp				; TAKT an PORTC ausgeben (zu Testzwecken)
; ----------------------------------
	; TAKT_NR gleich NULL?	
	LDS temp,(adr_HF_TAKT_NR)	; laden			
	tst temp
	breq TIM0_OVF_FERTIG
; ----------------------------------
	; TAKT_NR ungleich EINS	
	LDS temp,(adr_HF_TAKT_NR)	; laden			
	cpi temp,1
	brne TIM0_OVF_START
; ----------------------------------
	; TAKT-PEGEL = HIGH		
	LDS temp,(adr_HF_TAKT_PEGEL); laden			
	tst temp
	breq TIM0_OVF_FERTIG
; ----------------------------------
TIM0_OVF_START:
	; SENDE-PROTOKOLL				
	rcall PROTOKOLL_HF

; ----------------------------------
	; HF_TAKT_NR +1			 	
	LDS temp,(adr_HF_TAKT_NR)	; laden			
	inc temp						; +1			
	STS(adr_HF_TAKT_NR),temp	; speichern		
; ----------------------------------
TIM0_OVF_FERTIG:
	pop ZH				; wiederherstellen	
	pop ZL
	pop	TEMP			; LESEN von SREG vom STACK (KOPIE)
	out	SREG,TEMP		; Wiederherstellen von SREG
	pop	TEMP			; Wiederherstellen von "TEMP" 			
reti
; ##############################################################################
; ##############################################################################
; ##############################################################################
; Hier wird das Sendeprotokoll definiert (Manchester)							
PROTOKOLL_HF:
	; Z-Zeiger initialisieren	
	ldi ZL,LOW (PROTOKOLL_HF_SPRINGMARKE)
	ldi ZH,HIGH(PROTOKOLL_HF_SPRINGMARKE)
	; + TAKT NR					
	LDS temp,(adr_HF_TAKT_NR)	; laden			
	add ZL, temp
	adc ZH,NULL
	; SPRUNG nach Z
	IJMP

PROTOKOLL_HF_SPRINGMARKE:
;-------- PROTOKOLL AUFBAU ---------		
	; SYNCHRONISATION			
	nop
	rjmp SYNCHRONISATION_BIT		; ein
	rjmp SYNCHRONISATION_BIT		; aus
	rjmp SYNCHRONISATION_BIT		; ein
	rjmp SYNCHRONISATION_BIT		; aus
	rjmp SYNCHRONISATION_BIT		; ein
	rjmp SYNCHRONISATION_BIT		; aus

	; DATEN-BYTE				
	rjmp DATEN_BIT_0
	rjmp DATEN_BIT_0
	rjmp DATEN_BIT_1
	rjmp DATEN_BIT_1
	rjmp DATEN_BIT_2
	rjmp DATEN_BIT_2
	rjmp DATEN_BIT_3
	rjmp DATEN_BIT_3
	rjmp DATEN_BIT_4
	rjmp DATEN_BIT_4
	rjmp DATEN_BIT_5
	rjmp DATEN_BIT_5
	rjmp DATEN_BIT_6
	rjmp DATEN_BIT_6
	rjmp DATEN_BIT_7
	rjmp DATEN_BIT_7
	; PAKET ENDE				
	rjmp PROTOKOLL_HF_AUS		; aus
	rjmp PROTOKOLL_HF_AUS		; aus
	rjmp PROTOKOLL_HF_AUS		; aus
	rjmp PROTOKOLL_HF_AUS		; aus
	rjmp PROTOKOLL_HF_AUS		; aus
	rjmp PROTOKOLL_HF_ENDE		; ENDE
;-------------------------------------------
SYNCHRONISATION_BIT:
	cbi PORTD,LED_D_GELB	; GELB  
	ldi temp,1
	rjmp BIT_AUSWERTUNG
;-------------------------------------------
DATEN_BIT_0:
	LDS temp,(adr_HF_DATENBYTE)
	andi temp,0b00000001
	rjmp BIT_AUSWERTUNG
DATEN_BIT_1:
	LDS temp,(adr_HF_DATENBYTE)
	andi temp,0b00000010
	rjmp BIT_AUSWERTUNG
DATEN_BIT_2:
	LDS temp,(adr_HF_DATENBYTE)
	andi temp,0b00000100
	rjmp BIT_AUSWERTUNG
DATEN_BIT_3:
	LDS temp,(adr_HF_DATENBYTE)
	andi temp,0b00001000
	rjmp BIT_AUSWERTUNG
DATEN_BIT_4:
	LDS temp,(adr_HF_DATENBYTE)
	andi temp,0b00010000
	rjmp BIT_AUSWERTUNG
DATEN_BIT_5:
	LDS temp,(adr_HF_DATENBYTE)
	andi temp,0b00100000
	rjmp BIT_AUSWERTUNG
DATEN_BIT_6:
	LDS temp,(adr_HF_DATENBYTE)
	andi temp,0b01000000
	rjmp BIT_AUSWERTUNG
DATEN_BIT_7:
	LDS temp,(adr_HF_DATENBYTE)
	andi temp,0b10000000
	rjmp BIT_AUSWERTUNG


;-------------------------------------------
BIT_AUSWERTUNG:
	; BIT gleich NULL ?
	tst temp
	breq BIT_AUSWERTUNG_LOW
	rjmp BIT_AUSWERTUNG_HIGH

BIT_AUSWERTUNG_HIGH:
	; TAKT gleich NULL
	LDS temp,(adr_HF_TAKT_PEGEL)
	tst temp
	breq PROTOKOLL_HF_AUS
	rjmp PROTOKOLL_HF_EIN
BIT_AUSWERTUNG_LOW:
	; TAKT gleich NULL
	LDS temp,(adr_HF_TAKT_PEGEL)
	tst temp
	breq PROTOKOLL_HF_EIN
	rjmp PROTOKOLL_HF_AUS

;-------------------------------------------
PROTOKOLL_HF_EIN:
	; PIN auf HIGH	
	sbi TX_433MHZ_PORT,TX_433MHZ_PIN
	ret
;-------------------------------------------
PROTOKOLL_HF_AUS:
	; PIN auf LOW	
	cbi TX_433MHZ_PORT,TX_433MHZ_PIN
	ret
;-------------------------------------------
PROTOKOLL_HF_ENDE:
	; PIN auf LOW	
	cbi TX_433MHZ_PORT,TX_433MHZ_PIN
	; TAKT NR auf 255
	STS(adr_HF_TAKT_NR),VOLL	; speichern		
	; GELB aus		
	sbi PORTD,LED_D_GELB
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
;																				
; INP: temp																		
;																				
; rcall SENDEN_HF	; INP: temp													
;																				
SENDEN_HF:
	push temp					; sichern			
SENDEN_HF_w:
	LDS temp,(adr_HF_TAKT_NR)	; laden				
	; ist NULL ?	
	tst temp
	brne SENDEN_HF_w
	pop temp					; wiederherstellen	
	; SENDE-BYTE	
	STS(adr_HF_DATENBYTE),temp
	; TAKT-NR		
	STS(adr_HF_TAKT_NR),EINS
ret
; ##############################################################################
; ##############################################################################
; ##############################################################################
